# Generated by Django 4.2.11 on 2024-09-02 10:21

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('capeditor', '0008_remove_capsetting_audience_types'),
    ]

    operations = [
        migrations.AddField(
            model_name='capsetting',
            name='un_country_boundary_geojson',
            field=models.JSONField(blank=True, null=True, verbose_name='UN Country Boundary GeoJSON'),
        ),
    ]
