# Generated by Django 4.2.7 on 2024-06-03 12:04

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('capeditor', '0007_alter_capsetting_wmo_oid_and_more'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='capsetting',
            name='audience_types',
        ),
    ]
