# Generated by Django 4.2.11 on 2024-03-21 05:25

import django.contrib.gis.db.models.fields
from django.db import migrations, models
import django.db.models.deletion
import modelcluster.fields


class Migration(migrations.Migration):

    dependencies = [
        ('capeditor', '0003_capsetting_logo'),
    ]

    operations = [
        migrations.CreateModel(
            name='PredefinedAlertArea',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('sort_order', models.IntegerField(blank=True, editable=False, null=True)),
                ('name', models.CharField(max_length=255, verbose_name='Name')),
                ('geom', django.contrib.gis.db.models.fields.MultiPolygonField(srid=4326, verbose_name='Area')),
                ('setting', modelcluster.fields.ParentalKey(on_delete=django.db.models.deletion.PROTECT, related_name='predefined_alert_areas', to='capeditor.capsetting')),
            ],
            options={
                'verbose_name': 'Predefined Area',
                'verbose_name_plural': 'Predefined Areas',
            },
        ),
    ]
