# Generated by Django 4.2.11 on 2024-06-03 11:47

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('capeditor', '0006_capsetting_wmo_oid'),
    ]

    operations = [
        migrations.AlterField(
            model_name='capsetting',
            name='wmo_oid',
            field=models.CharField(blank=True, help_text='WMO Register of Alerting Authorities Object Identifier (OID) of the sending institution. This will be used to generate CAP messages identifiers', max_length=255, null=True, verbose_name='WMO Register of Alerting Authorities OID'),
        ),
        migrations.AlterField(
            model_name='hazardeventtypes',
            name='event',
            field=models.CharField(help_text='Name of Hazard', max_length=35, unique=True, verbose_name='Hazard'),
        ),
    ]
