# Generated by Django 4.2.11 on 2024-05-30 13:21

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('capeditor', '0005_alter_capsetting_sender'),
    ]

    operations = [
        migrations.AddField(
            model_name='capsetting',
            name='wmo_oid',
            field=models.CharField(blank=True, max_length=255, null=True, verbose_name='WMO Register of Alerting Authorities OID'),
        ),
    ]
