# Generated by Django 4.1.10 on 2023-10-13 14:25

from django.db import migrations, models
import django.db.models.deletion
import modelcluster.fields


class Migration(migrations.Migration):

    dependencies = [
        ('capeditor', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='capsetting',
            options={'verbose_name': 'CAP Settings'},
        ),
        migrations.RemoveField(
            model_name='capsetting',
            name='hazard_types',
        ),
        migrations.CreateModel(
            name='HazardEventTypes',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('sort_order', models.IntegerField(blank=True, editable=False, null=True)),
                ('is_in_wmo_event_types_list', models.BooleanField(default=True, verbose_name='Select from WMO list of Hazards Event Types')),
                ('event', models.CharField(help_text='Name of Hazard', max_length=255, unique=True, verbose_name='Hazard')),
                ('icon', models.CharField(blank=True, help_text='Matching icon', max_length=255, null=True, verbose_name='Icon')),
                ('setting', modelcluster.fields.ParentalKey(on_delete=django.db.models.deletion.PROTECT, related_name='hazard_event_types', to='capeditor.capsetting')),
            ],
            options={
                'ordering': ['sort_order'],
                'abstract': False,
            },
        ),
    ]
